@@ -36,7 +36,7 @@ static const char sccsid[] = "@(#)ant3d.c	5.13 2004/07/23 xlockmore";
 #define ant3d_opts xlockmore_opts
 #define DEFAULTS "*delay: 5000 \n" \
  "*count: -3 \n" \
- "*cycles: 100000 \n" \
+ "*cycles: 10000 \n" \
  "*ncolors: 200 \n" \
  "*wireframe: False \n" \
  "*fullrandom: False \n" \
@@ -98,7 +98,7 @@ ModeSpecOpt ant3d_opts =
 ModStruct   ant3d_description =
 {"ant3d", "init_ant3d", "draw_ant3d", "release_ant3d",
  "refresh_ant3d", "init_ant3d", NULL, &ant3d_opts,
- 5000, -3, 100000, 1, 64, 1.0, "",
+ 5000, -3, 10000, 1, 64, 1.0, "",
  "Shows 3D ants", 0, NULL};
 
 #endif
@@ -128,6 +128,11 @@ ModStruct   ant3d_description =
 
 #define SetList3D(x,y,z,c) if (!SetMem(ap,(unsigned int)x,(unsigned int)y,(unsigned int)z,c)) return False; \
 if (!AddToList(ap,(unsigned int)x,(unsigned int)y,(unsigned int)z,c)) return False
+#define InitSetList3D(x,y,z,c) if (!SetMem(ap,(unsigned int)x,(unsigned int)y,(unsigned int)z,c)) return; \
+if (!AddToList(ap,(unsigned int)x,(unsigned int)y,(unsigned int)z,c)) return
+
+#define ChangeList3D(i,x,y,z,c) if (!SetMem(ap,(unsigned int)x,(unsigned int)y,(unsigned int)z,c)) return False; \
+ChangeInList(ap,i,(unsigned int)x,(unsigned int)y,(unsigned int)z,c)
 
 #define EyeToScreen 72.0	/* distance from eye to screen */
 #define HalfScreenD 20.0	/* 1/2 the diameter of screen */
@@ -173,6 +178,7 @@ typedef struct {
 	double      azm;
 	double      metaAlt, metaAzm, metaDist;
 	antstruct  *ants;
+	antstruct  **antList;
 	antstruct  *ptrhead, *ptrend, eraserhead, eraserend;
 	antstruct  *buckethead[NBUCKETS], *bucketend[NBUCKETS];
 	Bool        wireframe, eyes;
@@ -431,129 +437,129 @@ static unsigned char tables[][3 * NCOLORS * STATES + 2] =
 {
 #if 0
 /* Generated */
-	{			/* 0: RLU */
+	{		/* 0: RLU */
 		3, 1,
 		1, TRS, 0, 2, TLS, 0, 0, TUS, 0
 	},
-	{			/* 1: RUL */
+	{		/* 1: RUL */
 		3, 1,
 		1, TRS, 0, 2, TUS, 0, 0, TLS, 0
 	},
-	{			/* 2: RUD */
+	{		/* 2: RUD */
 		3, 1,
 		1, TRS, 0, 2, TUS, 0, 0, TDS, 0
 	},
-	{			/* 3: RLUU */
+	{		/* 3: RLUU */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TUS, 0, 0, TUS, 0
 	},
-	{			/* 4: RRLU 32 */
+	{		/* 4: RRLU 32 */
 		4, 1,
 		1, TRS, 0, 2, TRS, 0, 3, TLS, 0, 0, TUS, 0
 	},
-	{			/* 5: RRUL ? */
+	{		/* 5: RRUL ? */
 		4, 1,
 		1, TRS, 0, 2, TRS, 0, 3, TUS, 0, 0, TLS, 0
 	},
-	{			/* 6: RRUD 22 */
+	{		/* 6: RRUD 22 */
 		4, 1,
 		1, TRS, 0, 2, TRS, 0, 3, TUS, 0, 0, TDS, 0
 	},
-	{			/* 7: RLRU 188 */
+	{		/* 7: RLRU 188 */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TRS, 0, 0, TUS, 0
 	},
-	{			/* 8: RLLU ? */
+	{		/* 8: RLLU ? */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TLS, 0, 0, TUS, 0
 	},
-	{			/* 9: RLUR 46 ! */
+	{		/* 9: RLUR 46 ! */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TUS, 0, 0, TRS, 0
 	},
-	{			/* 10: RLUL 32 */
+	{		/* 10: RLUL 32 */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TUS, 0, 0, TLS, 0
 	},
-	{			/* 11: RLUU 102 */
+	{		/* 11: RLUU 102 */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TUS, 0, 0, TUS, 0
 	},
-	{			/* 12: RLUD 28 */
+	{		/* 12: RLUD 28 */
 		4, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TUS, 0, 0, TDS, 0
 	},
-	{			/* 13: RURL 30 */
+	{		/* 13: RURL 30 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TRS, 0, 0, TLS, 0
 	},
-	{			/* 14: RURD ? */
+	{		/* 14: RURD ? */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TRS, 0, 0, TDS, 0
 	},
-	{			/* 15: RULL 22 */
+	{		/* 15: RULL 22 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TLS, 0, 0, TLS, 0
 	},
-	{			/* 16: RULR 22 */
+	{		/* 16: RULR 22 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TLS, 0, 0, TRS, 0
 	},
-	{			/* 17: RULU 22 */
+	{		/* 17: RULU 22 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TLS, 0, 0, TUS, 0
 	},
-	{			/* 18: RUUL 26 */
+	{		/* 18: RUUL 26 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TUS, 0, 0, TLS, 0
 	},
-	{			/* 19: RULD 22 */
+	{		/* 19: RULD 22 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TLS, 0, 0, TDS, 0
 	},
-	{			/* 20: RUUD 22 */
+	{		/* 20: RUUD 22 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TUS, 0, 0, TDS, 0
 	},
-	{			/* 21: RUDR 14 */
+	{		/* 21: RUDR 14 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TDS, 0, 0, TRS, 0
 	},
-	{			/* 22: RUDL ? */
+	{		/* 22: RUDL ? */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TDS, 0, 0, TLS, 0
 	},
-	{			/* 23: RUDU 114 */
+	{		/* 23: RUDU 114 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TDS, 0, 0, TUS, 0
 	},
-	{			/* 24: RUDD 40 */
+	{		/* 24: RUDD 40 */
 		4, 1,
 		1, TRS, 0, 2, TUS, 0, 3, TDS, 0, 0, TDS, 0
 	},
-	{			/* 25: RRRULL ? */
+	{		/* 25: RRRULL ? */
 		6, 1,
 		1, TRS, 0, 2, TRS, 0, 3, TRS, 0, 4, TUS, 0,
-		5, TLS, 0, 0, TLS, 0,
-	}
-	{			/* 26: RLRUUUL 25436 */
+		5, TLS, 0, 0, TLS, 0
+	},
+	{		/* 26: RLRUUUL 25436 */
 		7, 1,
 		1, TRS, 0, 2, TLS, 0, 3, TRS, 0, 4, TUS, 0,
-		5, TUS, 0, 6, TUS, 0, 0, TLS, 0,
+		5, TUS, 0, 6, TUS, 0, 0, TLS, 0
 	},
 #endif
-	{			/* 27: RRLDDDULRRLLLL * */
+	{		/* 27: RRLDDDULRRLLLL * */
 		14, 1,
 		1, TRS, 0, 2, TRS, 0, 3, TLS, 0, 4, TDS, 0,
 		5, TDS, 0, 6, TDS, 0, 7, TUS, 0, 8, TLS, 0,
 		9, TRS, 0, 10, TRS, 0, 11, TLS, 0, 12, TLS, 0,
-		13, TLS, 0, 0, TLS, 0,
+		13, TLS, 0, 0, TLS, 0
 	}
 };
 
 #define NTABLES   (sizeof tables / sizeof tables[0])
 
-#ifdef DEBUG
+#ifdef gDEBUG
 static char * printDir(int dir, Bool omega)
 {
 	if (omega) {
@@ -785,6 +791,17 @@ AddToList(antfarm3dstruct * ap,
 	return True;
 }
 
+static void
+ChangeInList(antfarm3dstruct * ap, int i,
+		unsigned int x, unsigned int y, unsigned int z, unsigned int c)
+{
+	antstruct  *ant = ap->antList[i];
+	ant->col = x;
+	ant->row = y;
+	ant->stack = z;
+	ant->color = c;
+}
+
 static void
 AddToEraseList(antfarm3dstruct * ap, antstruct * cell)
 {
@@ -812,7 +829,12 @@ DelFromList(antfarm3dstruct * ap, antstruct * cell)
 		if (ap->ptrend != NULL)
 			ap->ptrend->next = NULL;
 	}
+}
 
+static void
+EraseFromList(antfarm3dstruct * ap, antstruct * cell)
+{
+	DelFromList(ap, cell);
 	AddToEraseList(ap, cell);
 }
 
@@ -904,7 +926,7 @@ End3D(antfarm3dstruct * ap)
 	while (ap->ptrhead != NULL) {
 		/* Reset3D(ap->ptrhead->col, ap->ptrhead->row,
 			ap->ptrhead->stack); */
-		DelFromList(ap, ap->ptrhead);
+		EraseFromList(ap, ap->ptrhead);
 	}
 	ptr = ap->eraserhead.next;
 	while (ptr != &ap->eraserend) {
@@ -914,6 +936,101 @@ End3D(antfarm3dstruct * ap)
 	MemInit(ap);
 }
 
+#if 0
+/* This does not seem to be needed */
+static void
+condenseList(antfarm3dstruct * ap) {
+	antstruct  *ptr1 = ap->ptrhead;
+	antstruct  *ptr2 = ap->ptrhead;
+	antstruct  *tmp = NULL;
+	while (ptr2->next != NULL) {
+		ptr2 = ptr2->next;
+		while (ptr1 != ptr2) {
+			if (ptr1->col == ptr2->col &&
+					ptr1->row == ptr2->row &&
+					ptr1->stack == ptr2->stack) {
+				int found = 0;
+				int i;
+
+				for (i = 0; i < ap->n; i++) {
+					if (ap->antList[i] == ptr1) {
+						found = 1;
+						break;
+					}
+				}
+				if (found == 0)
+					tmp = ptr1;
+			}
+			ptr1 = ptr1->next;
+			if (tmp != NULL) {
+				DelFromList(ap, tmp);
+				free(tmp);
+				tmp = NULL;
+			}
+		}
+	}
+}
+#endif
+
+#ifndef BLACK_CELLS
+static void
+eraseEmpty (antfarm3dstruct * ap) {
+	antstruct  *ptr1 = ap->ptrhead;
+	antstruct  *ptr2 = ap->ptrhead;
+	antstruct  *tmp = NULL;
+	int col, row, stack;
+	while (ptr1->next != NULL) {
+		if (ptr1->color == 0) {
+			int found = 0;
+			int i;
+
+			for (i = 0; i < ap->n; i++) {
+				if (ap->antList[i] == ptr1) {
+					found = 1;
+					break;
+				}
+			}
+			if (found == 0)
+				tmp = ptr1;
+		}
+		col = ptr1->col;
+		row = ptr1->row;
+		stack = ptr1->stack;
+		ptr1 = ptr1->next;
+		if (tmp != NULL) {
+			DelFromList(ap, tmp);
+			free(tmp);
+			tmp = NULL;
+			/* may have to delete all in that position */
+			ptr2 = ap->ptrhead;
+			while (ptr1 != ptr2) {
+				if (ptr2->col == col &&
+						ptr2->row == row &&
+						ptr2->stack == stack) {
+					int found = 0;
+					int i;
+
+					for (i = 0; i < ap->n; i++) {
+						if (ap->antList[i] == ptr2) {
+							found = 1;
+							break;
+						}
+					}
+					if (found == 0)
+						tmp = ptr2;
+				}
+				ptr2 = ptr2->next;
+				if (tmp != NULL) {
+					DelFromList(ap, tmp);
+					free(tmp);
+					tmp = NULL;
+				}
+			}
+		}
+	}
+}
+#endif
+
 static Bool
 RunAnt3D(antfarm3dstruct * ap)
 {
@@ -938,7 +1055,7 @@ RunAnt3D(antfarm3dstruct * ap)
 			status->color, printTurn(ap, status->turn),
 			anant->state, ap->ncolors);
 #endif
-		SetList3D(anant->col, anant->row, anant->stack,
+		ChangeList3D(i, anant->col, anant->row, anant->stack,
 			status->color);
 		ap->tape[tape_pos] = status->color;     /* write on tape */
 #ifdef gDEBUG
@@ -958,7 +1075,12 @@ RunAnt3D(antfarm3dstruct * ap)
 			&(anant->col), &(anant->row), &(anant->stack));
 		SetList3D(anant->col, anant->row, anant->stack,
 			NCOLORS);
-        }
+		ap->antList[i] = ap->ptrend;
+	}
+	/*condenseList(ap);*/
+#ifndef BLACK_CELLS
+	eraseEmpty(ap);
+#endif
 	return True;
 }
 
@@ -1035,21 +1157,21 @@ getTurk(ModeInfo * mi, int logClass, int number)
 #ifdef DEBUG
 	(void) printf("logClass %d, number %d\n", logClass, number);
 #endif
-	power4 = 1 << (2 * (logClass - 1));
+	power4 = 1 << ((logClass - 1) << 1);
 	ap->ncolors = logClass;
 	ap->nstates = 1;
 	total = ap->ncolors * ap->nstates;
 	for (j = 0; j < total; j++) {
 		ap->machine[j].color = (j + 1) % total;
 		ap->machine[j].turn = ((power4 * 3) & number) / power4;
-			rule[j] = ap->machine[j].turn + '0';
+		rule[j] = ap->machine[j].turn + '0';
 		ap->machine[j].next = 0;
 		power4 >>= 2;
 	}
 	rule[total] = '\0';
 	if (MI_IS_VERBOSE(mi))
 		(void) fprintf(stdout,
-	 		"ants %d, Turk's number 0x%x (%s), colors %d\n",
+	 		"ants %d, Turk's number 0x%x (0%s)B4, colors %d\n",
 			ap->n, number, rule, ap->ncolors);
 	if (label)
 		(void) sprintf(ap->ruleString, "%d Turk's number 0x%x",
@@ -1132,19 +1254,31 @@ parseRule(ModeInfo * mi)
 			(!bucket[0] && !bucket[3]) || (!bucket[1] && !bucket[2]) ||
 			(!bucket[1] && !bucket[3]) || (!bucket[2] && !bucket[3])) {
 			if (ap->neighbors == 12) {
-				switch (NRAND(3)) {
+				switch (NRAND(5)) {
 				case 0: getTurk(mi, 3, 0x12);
 					break;
-				case 1: getTurk(mi, 3, 0x18);
+				case 1: getTurk(mi, 3, 0x13);
+					break;
+				case 2: getTurk(mi, 3, 0x18);
+					break;
+				case 3: getTurk(mi, 3, 0x1b);
 					break;
 				default: getTurk(mi, 4, 0x47);
 					break;
 				}
 			} else {
-				switch (NRAND(3)) {
-				case 0: getTurk(mi, 3, 0x21);
+				switch (NRAND(7)) {
+				case 0: getTurk(mi, 3, 0x12);
 					break;
-				case 1: getTurk(mi, 3, 0x27);
+				case 1: getTurk(mi, 3, 0x13);
+					break;
+				case 2: getTurk(mi, 3, 0x18);
+					break;
+				case 3: getTurk(mi, 3, 0x1b);
+					break;
+				case 4: getTurk(mi, 3, 0x21);
+					break;
+				case 5: getTurk(mi, 3, 0x27);
 					break;
 				default: getTurk(mi, 3, 0x2d);
 					break;
@@ -1372,7 +1506,7 @@ DrawEyes(ModeInfo * mi, int color, XPoint * pts,
 }
 
 static void
-DrawFace(ModeInfo * mi, int dir, int color, antstruct * cell, XPoint * pts,
+drawQuad(ModeInfo * mi, int dir, int color, antstruct * cell, XPoint * pts,
 		int p1, int p2, int p3, int p4)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -1451,12 +1585,11 @@ DrawCube(ModeInfo * mi, antstruct * cell)
 	unsigned int mask;
 	double x, y, z;
 	double dx, dy, dz;
-	int color;
+	int color = cell->color;
 
 	x = (double) cell->col - ap->ox;
 	y = (double) cell->row - ap->oy;
 	z = (double) cell->stack - ap->oz;
-	color = cell->color;
 	for (dz = z - LEN; dz <= z + LEN2; dz += LEN2)
 		for (dy = y - LEN; dy <= y + LEN2; dy += LEN2)
 			for (dx = x - LEN; dx <= x + LEN2; dx += LEN2) {
@@ -1491,30 +1624,30 @@ DrawCube(ModeInfo * mi, antstruct * cell)
 	*/
 	if (ap->wireframe) {
 		if (dz <= LEN)
-			DrawFace(mi, -1, (int) (color & mask), cell, pts, 4, 5, 7, 6);
+			drawQuad(mi, -1, (int) (color & mask), cell, pts, 4, 5, 7, 6);
 		else if (dz >= -LEN)
-			DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 1, 3, 2);
+			drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 1, 3, 2);
 		if (dx <= LEN)
-			DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 3, 7, 5);
+			drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 3, 7, 5);
 		else if (dx >= -LEN)
-			DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 2, 6, 4);
+			drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 2, 6, 4);
 		if (dy <= LEN)
-			DrawFace(mi, -1, (int) (color & mask), cell, pts, 2, 3, 7, 6);
+			drawQuad(mi, -1, (int) (color & mask), cell, pts, 2, 3, 7, 6);
 		else if (dy >= -LEN)
-			DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 1, 5, 4);
+			drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 1, 5, 4);
 	}
 	if (dz > LEN)
-		DrawFace(mi, ZP, (int) (color & mask), cell, pts, 4, 5, 7, 6);
+		drawQuad(mi, ZP, (int) (color & mask), cell, pts, 4, 5, 7, 6);
 	else if (dz < -LEN)
-		DrawFace(mi, ZN, (int) (color & mask), cell, pts, 0, 1, 3, 2);
+		drawQuad(mi, ZN, (int) (color & mask), cell, pts, 0, 1, 3, 2);
 	if (dx > LEN)
-		DrawFace(mi, XP, (int) (color & mask), cell, pts, 1, 3, 7, 5);
+		drawQuad(mi, XP, (int) (color & mask), cell, pts, 1, 3, 7, 5);
 	else if (dx < -LEN)
-		DrawFace(mi, XN, (int) (color & mask), cell, pts, 0, 2, 6, 4);
+		drawQuad(mi, XN, (int) (color & mask), cell, pts, 0, 2, 6, 4);
 	if (dy > LEN)
-		DrawFace(mi, YP, (int) (color & mask), cell, pts, 2, 3, 7, 6);
+		drawQuad(mi, YP, (int) (color & mask), cell, pts, 2, 3, 7, 6);
 	else if (dy < -LEN)
-		DrawFace(mi, YN, (int) (color & mask), cell, pts, 0, 1, 5, 4);
+		drawQuad(mi, YN, (int) (color & mask), cell, pts, 0, 1, 5, 4);
 	return (1);
 }
 
@@ -1600,37 +1733,37 @@ DrawRhombicDodecahedron(ModeInfo * mi, antstruct * cell)
 	*/
 	if (ap->wireframe) {
 		if (dx <= LEN) {
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 3, 2, 5);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 6, 13, 4);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 4, 1, 3);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 5, 12, 6);
 		} else if (dx >= -LEN) {
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 2, 7, 11, 9);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 8, 11, 7);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 8, 13, 10, 11);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 9, 11, 10, 12);
 		}
 		if (dy <= LEN) {
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 4, 1, 3);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 8, 11, 7);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 7, 2, 3);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 4, 13, 8);
 		} else if (dy >= -LEN) {
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 5, 12, 6);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 9, 11, 10, 12);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 2, 9, 12, 5);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 6, 12, 10, 13);
 		}
 		if (dz <= LEN) {
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 7, 2, 3);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 2, 9, 12, 5);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 2, 7, 11, 9);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 3, 2, 5);
 		} else if (dz >= -LEN) {
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 1, 4, 13, 8);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 6, 12, 10, 13);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 0, 6, 13, 4);
-		DrawFace(mi, -1, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		drawQuad(mi, -1, (int) (color & mask), cell, pts, 8, 13, 10, 11);
 		}
 	}
 	sortx = sorty = sortz = 0;
@@ -1651,50 +1784,50 @@ DrawRhombicDodecahedron(ModeInfo * mi, antstruct * cell)
 	    if (dx > LEN) {
 	      if (sortz > sorty) {
 		if (dz < 0)
-		  DrawFace(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		  drawQuad(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
 		else
-		  DrawFace(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
-		DrawFace(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
-		DrawFace(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		  drawQuad(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		drawQuad(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		drawQuad(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
 		if (dz < 0)
-		  DrawFace(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		  drawQuad(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
 		else
-		  DrawFace(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		  drawQuad(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
 	      } else {
 		if (dy < 0)
-		  DrawFace(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		  drawQuad(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
 		else
-		  DrawFace(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
-		DrawFace(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
-		DrawFace(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		  drawQuad(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		drawQuad(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		drawQuad(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
 		if (dy < 0)
-		  DrawFace(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		  drawQuad(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
 		else
-		  DrawFace(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		  drawQuad(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
 	      }
 	    } else if (dx < -LEN) {
 	      if (sortz > sorty) {
 		if (dz < 0)
-		  DrawFace(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		  drawQuad(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
 		else
-		  DrawFace(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
-		DrawFace(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
-		DrawFace(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		  drawQuad(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		drawQuad(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		drawQuad(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
 		if (dz < 0)
-		  DrawFace(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		  drawQuad(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
 		else
-		  DrawFace(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		  drawQuad(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
 	      } else {
 		if (dy < 0)
-		  DrawFace(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		  drawQuad(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
 		else
-	  	  DrawFace(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
-		DrawFace(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
-		DrawFace(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+	  	  drawQuad(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		drawQuad(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		drawQuad(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
 		if (dy < 0)
-	  	  DrawFace(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+	  	  drawQuad(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
 		else
-		  DrawFace(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		  drawQuad(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
 	      }
 	    }
 	  }
@@ -1702,50 +1835,50 @@ DrawRhombicDodecahedron(ModeInfo * mi, antstruct * cell)
 	    if (dy > LEN) {
 	      if (sortx > sortz) {
 		if (dx < 0)
-		  DrawFace(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		  drawQuad(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
 		else
-		  DrawFace(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
-		DrawFace(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
-		DrawFace(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		  drawQuad(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		drawQuad(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		drawQuad(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
 		if (dx < 0)
-		  DrawFace(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		  drawQuad(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
 		else
-		  DrawFace(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		  drawQuad(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
 	      } else {
 		if (dz < 0)
-		  DrawFace(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		  drawQuad(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
 		else
-		  DrawFace(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
-		DrawFace(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
-		DrawFace(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
+		  drawQuad(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		drawQuad(mi, XNYP, (int) (color & mask), cell, pts, 1, 8, 11, 7);
+		drawQuad(mi, XPYP, (int) (color & mask), cell, pts, 0, 4, 1, 3);
 		if (dz < 0)
-		  DrawFace(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		  drawQuad(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
 		else
-		  DrawFace(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		  drawQuad(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
 	      }
 	    } else if (dy < -LEN) {
 	      if (sortx > sortz) {
 		if (dx < 0)
-		  DrawFace(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		  drawQuad(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
 		else
-		  DrawFace(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
-		DrawFace(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
-		DrawFace(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		  drawQuad(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		drawQuad(mi, YNZP, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		drawQuad(mi, YNZN, (int) (color & mask), cell, pts, 2, 9, 12, 5);
 		if (dx < 0)
-		  DrawFace(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		  drawQuad(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
 		else
-		  DrawFace(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		  drawQuad(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
 	      } else {
 		if (dz < 0)
-		  DrawFace(mi, YNZN, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		  drawQuad(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
 		else
-		  DrawFace(mi, YNZP, (int) (color & mask), cell, pts, 6, 12, 10, 13);
-		DrawFace(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
-		DrawFace(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
+		  drawQuad(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		drawQuad(mi, XNYN, (int) (color & mask), cell, pts, 9, 11, 10, 12);
+		drawQuad(mi, XPYN, (int) (color & mask), cell, pts, 0, 5, 12, 6);
 		if (dz < 0)
-		  DrawFace(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		  drawQuad(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
 		else
-		  DrawFace(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		  drawQuad(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
 	      }
 	    }
 	  }
@@ -1753,50 +1886,50 @@ DrawRhombicDodecahedron(ModeInfo * mi, antstruct * cell)
 	    if (dz > LEN) {
 	      if (sorty > sortx) {
 		if (dy < 0)
-		  DrawFace(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		  drawQuad(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
 		else
-		  DrawFace(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
-		DrawFace(mi, ZPXN, (int) (color & mask), cell, pts, 0, 3, 2, 5);
-		DrawFace(mi, ZPXP, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		  drawQuad(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		drawQuad(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		drawQuad(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
 		if (dy < 0)
-		  DrawFace(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		  drawQuad(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
 		else
-		  DrawFace(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		  drawQuad(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
 	      } else {
 		if (dx < 0)
-		  DrawFace(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		  drawQuad(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
 		else
-		  DrawFace(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
-		DrawFace(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
-		DrawFace(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
+		  drawQuad(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		drawQuad(mi, YNZP, (int) (color & mask), cell, pts, 2, 9, 12, 5);
+		drawQuad(mi, YPZP, (int) (color & mask), cell, pts, 1, 7, 2, 3);
 		if (dx < 0)
-		  DrawFace(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
+		  drawQuad(mi, ZPXN, (int) (color & mask), cell, pts, 2, 7, 11, 9);
 		else
-		  DrawFace(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
+		  drawQuad(mi, ZPXP, (int) (color & mask), cell, pts, 0, 3, 2, 5);
 	      }
 	    } else if (dz < -LEN) {
 	      if (sorty > sortx) {
 		if (dy < 0)
-		  DrawFace(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		  drawQuad(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
 		else
-		  DrawFace(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
-		DrawFace(mi, ZNXN, (int) (color & mask), cell, pts, 1, 8, 13, 10);
-		DrawFace(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		  drawQuad(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		drawQuad(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		drawQuad(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
 		if (dy < 0)
-		  DrawFace(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		  drawQuad(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
 		else
-		  DrawFace(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		  drawQuad(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
 	      } else {
 		if (dx < 0)
-		  DrawFace(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		  drawQuad(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
 		else
-		  DrawFace(mi, ZNXN, (int) (color & mask), cell, pts, 1, 8, 13, 10);
-		DrawFace(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
-		DrawFace(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
+		  drawQuad(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
+		drawQuad(mi, YNZN, (int) (color & mask), cell, pts, 6, 12, 10, 13);
+		drawQuad(mi, YPZN, (int) (color & mask), cell, pts, 1, 4, 13, 8);
 		if (dx < 0)
-		  DrawFace(mi, ZNXN, (int) (color & mask), cell, pts, 1, 8, 13, 10);
+		  drawQuad(mi, ZNXN, (int) (color & mask), cell, pts, 8, 13, 10, 11);
 		else
-		  DrawFace(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
+		  drawQuad(mi, ZNXP, (int) (color & mask), cell, pts, 0, 6, 13, 4);
 	      }
 	    }
 	  }
@@ -1902,6 +2035,10 @@ free_ant3d(Display *display, antfarm3dstruct *ap)
 		free(ap->tape);
 		ap->tape = (unsigned char *) NULL;
 	}
+	if (ap->antList != NULL) {
+		free(ap->antList);
+		ap->antList = (antstruct **) NULL;
+	}
 	if (ap->ants != NULL) {
 		free(ap->ants);
 		ap->ants = (antstruct *) NULL;
@@ -1983,7 +2120,7 @@ init_ant3d(ModeInfo * mi)
 		End3D(ap);
 	}
 	ap->labelOffsetX = NRAND(8);
-        ap->labelOffsetY = NRAND(8);
+	ap->labelOffsetY = NRAND(8);
 	parseRule(mi);
 	if (MI_NPIXELS(mi) > 2) {
 		int offset = NRAND(MI_NPIXELS(mi));
@@ -2024,27 +2161,38 @@ init_ant3d(ModeInfo * mi)
 			return;
 		}
 	}
+	if (ap->antList == NULL) {
+		if ((ap->antList = (antstruct **) malloc(ap->n *
+				sizeof (antstruct *))) == NULL) {
+			free_ant3d(display, ap);
+			return;
+		}
+	}
 
 	for (i = 0; i < ap->n; i++) {
-		ap->ants[i].row = ap->nrows / 2;
-		ap->ants[i].col = ap->ncols / 2;
-		ap->ants[i].stack = ap->nstacks / 2;
-		ap->ants[i].direction = NRAND((ap->neighbors == 12) ? 12 : 6);
-		ap->ants[i].gravity = NRAND(6);
+		antstruct *anant = &ap->ants[i];
+		anant->col = ap->ncols / 2;
+		anant->row = ap->nrows / 2;
+		anant->stack = ap->nstacks / 2;
+		anant->direction = NRAND((ap->neighbors == 12) ? 12 : 6);
+		anant->gravity = NRAND(6);
 		if (ap->neighbors == 12) {
-			if (ap->ants[i].direction / 4 == 0) {
-				ap->ants[i].gravity = NRAND(2) + ZN;
-			} else if (ap->ants[i].direction / 4 == 1) {
-				ap->ants[i].gravity = NRAND(2) + XN;
+			if (anant->direction / 4 == 0) {
+				anant->gravity = NRAND(2) + ZN;
+			} else if (anant->direction / 4 == 1) {
+				anant->gravity = NRAND(2) + XN;
 			} else {
-				ap->ants[i].gravity = NRAND(2) + YN;
+				anant->gravity = NRAND(2) + YN;
 			}
 		} else {
-			while ((ap->ants[i].gravity / 2) % 3 ==
-					(ap->ants[i].direction / 2) % 3)
-				ap->ants[i].gravity = NRAND(6);
+			while ((anant->gravity / 2) % 3 ==
+					(anant->direction / 2) % 3)
+				anant->gravity = NRAND(6);
 		}
-		ap->ants[i].state = 0;
+		anant->state = 0;
+		InitSetList3D(anant->col, anant->row, anant->stack,
+			NCOLORS);
+		ap->antList[i] = ap->ptrend;
 	}
 	if (ap->dbuf != None) {
 		XFreePixmap(display, ap->dbuf);
@@ -2056,9 +2204,9 @@ init_ant3d(ModeInfo * mi)
 		return;
 	}
 	if (MI_IS_FULLRANDOM(mi)) {
-                ap->eyes = (Bool) (LRAND() & 1);
-        } else {
-                ap->eyes = eyes;
+		ap->eyes = (Bool) (LRAND() & 1);
+	} else {
+		ap->eyes = eyes;
 	}
 	DrawScreen(mi);
 }
@@ -2083,7 +2231,7 @@ draw_ant3d(ModeInfo * mi)
 	DrawScreen(mi);
 	MI_IS_DRAWN(mi) = True;
 
-	if (++ap->generation > MI_CYCLES(mi) || ap->nInvisible > 32) {
+	if (++ap->generation > MI_CYCLES(mi) / ap->n || ap->nInvisible > 32) {
 		/*CountCells3D(ap) == 0) */
 		init_ant3d(mi);
 	} else
