@@ -375,13 +375,19 @@ static dclockstruct *dclocks = (dclockstruct *) NULL;
 
 extern char *message;
 
-static unsigned long
-timeAtLastNewYear(long timeNow)
+static time_t
+timeAtLastNewYear(time_t timeNow)
 {
 	struct tm *t;
 
-	t = localtime((const time_t *) &timeNow);
-	return (unsigned long)(t->tm_year);
+	t = localtime(&timeNow);
+	t->tm_mon = 0;
+	t->tm_mday = 1;
+	t->tm_hour = 0;
+	t->tm_min = 0;
+	t->tm_sec = 0;
+	t->tm_isdst = -1; /* mktime shall decide itself */
+	return mktime(t);
 }
 
 #ifndef HAVE_SNPRINTF
@@ -420,7 +426,7 @@ convert(double x, char *string)
 }
 
 static void
-dayhrminsec(long timeCount, int tzoffset, char *string)
+dayhrminsec(time_t timeCount, int tzoffset, char *string)
 {
 	int days, hours, minutes, secs;
 	int bufsize, i;
@@ -675,7 +681,7 @@ drawDclock(ModeInfo * mi)
 				"%a %b %d %Y", localtime(&(dp->timeold)));
 		}
 	  } else {
-		long timeNow, timeLocal;
+		time_t timeNow, timeLocal;
 		timeNow = seconds();
 		timeLocal = timeNow + dp->tzoffset;
 
@@ -950,7 +956,7 @@ init_dclock(ModeInfo * mi)
 {
 	Display *display = MI_DISPLAY(mi);
 	dclockstruct *dp;
-	long timeNow, timeLocal;
+	time_t timeNow, timeLocal;
 	int i, j;
 
 	if (dclocks == NULL) {
@@ -989,7 +995,7 @@ init_dclock(ModeInfo * mi)
 #if (!defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)) || (!defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan)) || (defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan))
 #define NUM_DCLOCK_MODES 10
 #endif
-#if !defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan)
+#if (!defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan))
 #define NUM_DCLOCK_MODES 9
 #endif
 	if (!(message && *message)) {
@@ -1033,9 +1039,7 @@ init_dclock(ModeInfo * mi)
 #endif
 #endif
 				break;
-#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium)) ||
-(defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)) ||
-(defined(MODE_dclock_mayan) && defined(MODE_dclock_y2k))
+#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium)) || (defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)) || (defined(MODE_dclock_mayan) && defined(MODE_dclock_y2k))
 			case 10:
 #if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium))
 				dp->millennium = True;
@@ -1043,8 +1047,7 @@ init_dclock(ModeInfo * mi)
 				dp->mayan = True;
 #endif
 				break;
-#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) &&
-defined(MODE_dclock_mayan)
+#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan))
 			case 11:
 				dp->mayan = True;
 				break;
@@ -1252,7 +1255,7 @@ defined(MODE_dclock_mayan)
 			dayhrminsec(MAYAN_TIME_START - timeLocal, dp->tzoffset, dp->strnew[1]);
 			dp->strpta[1] = dp->strnew[1];
 		} else {
-			struct tm *t = localtime((const time_t *) &timeLocal);
+			struct tm *t = localtime(&timeLocal);
 
 			if (dp->time24)
 			  (void) strftime(dp->strnew[0], STRSIZE, "%H:%M:%S", t);
