@@ -103,11 +103,11 @@ static const char sccsid[] = "@(#)life3d.c	5.27 2008/07/28 xlockmore";
 #else
 #define DEF_RULE  "P"		/* All rules with known patterns */
 #define DEF_RULE  "B5/S45"
-#define DEF_RULE  "B6/S567"
 #define DEF_RULE  "B5/S56"
 #define DEF_RULE  "B5/S678"
+#define DEF_RULE  "B6/S567"
 #define DEF_RULE  "B67/S67"
-/* There are no known gliders for B67/S67 */
+/* There is a glider for B67/S67 but its kind of big */
 #endif
 
 #define DEF_GLIDERSEARCH  "False"
@@ -1512,18 +1512,18 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 				patptr = &patterns_14B4S34[pat][0];
 #ifdef LIFE_NAMES
 				patstrg = &patterns_14B4S34[2 * pattern][0];
-#endif
-				break;
-			case LIFE_14B45S56:
-				patptr = &patterns_14B45S56[pat][0];
-#ifdef LIFE_NAMES
-				patstrg = &patterns_14B45S56[2 * pattern][0];
 #endif
 				break;
 			case LIFE_14B46S34:
 				patptr = &patterns_14B46S34[pat][0];
 #ifdef LIFE_NAMES
 				patstrg = &patterns_14B46S34[2 * pattern][0];
+#endif
+				break;
+			case LIFE_14B45S56:
+				patptr = &patterns_14B45S56[pat][0];
+#ifdef LIFE_NAMES
+				patstrg = &patterns_14B45S56[2 * pattern][0];
 #endif
 				break;
 			}
@@ -1614,24 +1614,6 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 				patptr = &patterns_26B5S56[pat][0];
 #ifdef LIFE_NAMES
 				patstrg = &patterns_26B5S56[2 * pattern][0];
-#endif
-				break;
-			case LIFE_26B6S567:
-				patptr = &patterns_26B6S567[pat][0];
-#ifdef LIFE_NAMES
-				patstrg = &patterns_26B6S567[2 * pattern][0];
-#endif
-				break;
-			case LIFE_26B6S57:
-				patptr = &patterns_26B6S57[pat][0];
-#ifdef LIFE_NAMES
-				patstrg = &patterns_26B6S57[2 * pattern][0];
-#endif
-				break;
-			case LIFE_26B67S57:
-				patptr = &patterns_26B67S57[pat][0];
-#ifdef LIFE_NAMES
-				patstrg = &patterns_26B67S57[2 * pattern][0];
 #endif
 				break;
 			case LIFE_26B5S58:
@@ -1650,6 +1632,30 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 				patptr = &patterns_26B5S8[pat][0];
 #ifdef LIFE_NAMES
 				patstrg = &patterns_26B5S8[2 * pattern][0];
+#endif
+				break;
+			case LIFE_26B58S58:
+				patptr = &patterns_26B58S58[pat][0];
+#ifdef LIFE_NAMES
+				patstrg = &patterns_26B58S58[2 * pattern][0];
+#endif
+				break;
+			case LIFE_26B6S567:
+				patptr = &patterns_26B6S567[pat][0];
+#ifdef LIFE_NAMES
+				patstrg = &patterns_26B6S567[2 * pattern][0];
+#endif
+				break;
+			case LIFE_26B6S57:
+				patptr = &patterns_26B6S57[pat][0];
+#ifdef LIFE_NAMES
+				patstrg = &patterns_26B6S57[2 * pattern][0];
+#endif
+				break;
+			case LIFE_26B67S57:
+				patptr = &patterns_26B67S57[pat][0];
+#ifdef LIFE_NAMES
+				patstrg = &patterns_26B67S57[2 * pattern][0];
 #endif
 				break;
 			case LIFE_26B67S67:
@@ -2585,7 +2591,9 @@ orthogonal(life3dstruct * lp) /* or full diagonal */
 			((lp->patterned_rule == LIFE_26B5S58) &&
 			(NRAND(4) == 0)) ||
 			((lp->patterned_rule == LIFE_26B5S678) &&
-			(LRAND() & 1)));
+			(LRAND() & 1)) ||
+			((lp->patterned_rule == LIFE_26B58S58) &&
+			(NRAND(4) != 0)));
 	return False;
 }
 
@@ -2727,25 +2735,6 @@ populateGliders(life3dstruct * lp, int dim, int orth,
 			(void) populateGlider(lp, glider_26B5S56[2], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B6S567) {
-		(void) populateGlider(lp, glider_26B6S567[0], dim,
-			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B6S57) {
-		if (orth) {
-			if (LRAND() & 1) {
-				(void) populateGlider(lp, glider_26B6S57[1], dim,
-					hsp, vsp, asp, hoff, voff, aoff);
-			} else {
-				(void) populateGlider(lp, glider_26B6S57[2], dim,
-					hsp, vsp, asp, hoff, voff, aoff);
-			}
-		} else {
-			(void) populateGlider(lp, glider_26B6S57[0], dim,
-				hsp, vsp, asp, hoff, voff, aoff);
-		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B67S57) {
-		(void) populateGlider(lp, glider_26B67S57[0], dim,
-			hsp, vsp, asp, hoff, voff, aoff);
 	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S58) {
 		if (orth)
 			(void) populateGlider(lp, glider_26B5S58[4], dim,
@@ -2772,7 +2761,63 @@ populateGliders(life3dstruct * lp, int dim, int orth,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
 	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S8) {
-		(void) populateGlider(lp, glider_26B5S8[0], dim,
+		if (LRAND() & 1) {
+			(void) populateGlider(lp, glider_26B5S8[0], dim,
+				hsp, vsp, asp, hoff, voff, aoff);
+		} else {
+			if (LRAND() & 1) {
+				(void) populateGlider(lp, glider_26B5S8[1], dim,
+					hsp, vsp, asp, hoff, voff, aoff);
+			} else {
+				(void) populateGlider(lp, glider_26B5S8[2], dim,
+					hsp, vsp, asp, hoff, voff, aoff);
+			}
+		}
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B58S58) {
+		if (orth) {
+			if (LRAND() & 1) {
+				if (LRAND() & 1)
+					(void) populateGlider(lp, glider_26B58S58[1], dim,
+						hsp, vsp, asp, hoff, voff, aoff);
+				else
+					(void) populateGlider(lp, glider_26B58S58[2], dim,
+						hsp, vsp, asp, hoff, voff, aoff);
+			} else {
+				if (LRAND() & 1)
+					(void) populateGlider(lp, glider_26B58S58[3], dim,
+						hsp, vsp, asp, hoff, voff, aoff);
+				else if (NRAND(3))
+					(void) populateGlider(lp, glider_26B58S58[4], dim,
+						hsp, vsp, asp, hoff, voff, aoff);
+				else if (LRAND() & 1)
+					(void) populateGlider(lp, glider_26B58S58[5], dim,
+						hsp, vsp, asp, hoff, voff, aoff);
+				else
+					(void) populateGlider(lp, glider_26B58S58[6], dim,
+						hsp, vsp, asp, hoff, voff, aoff);
+			}
+		} else {
+			(void) populateGlider(lp, glider_26B58S58[0], dim,
+				hsp, vsp, asp, hoff, voff, aoff);
+		}
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B6S567) {
+		(void) populateGlider(lp, glider_26B6S567[0], dim,
+			hsp, vsp, asp, hoff, voff, aoff);
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B6S57) {
+		if (orth) {
+			if (LRAND() & 1) {
+				(void) populateGlider(lp, glider_26B6S57[1], dim,
+					hsp, vsp, asp, hoff, voff, aoff);
+			} else {
+				(void) populateGlider(lp, glider_26B6S57[2], dim,
+					hsp, vsp, asp, hoff, voff, aoff);
+			}
+		} else {
+			(void) populateGlider(lp, glider_26B6S57[0], dim,
+				hsp, vsp, asp, hoff, voff, aoff);
+		}
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B67S57) {
+		(void) populateGlider(lp, glider_26B67S57[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
 	}
 }
@@ -3050,6 +3095,7 @@ init_life3d(ModeInfo * mi)
 	}
 	lp->colors[0] = MI_BLACK_PIXEL(mi);
 	if (MI_NPIXELS(mi) > 2) {
+#ifdef DISTRIB_COLOR
 		i = NRAND(3);
 
 		lp->colors[i + 1] = MI_PIXEL(mi,
@@ -3060,6 +3106,15 @@ init_life3d(ModeInfo * mi)
 		lp->colors[(i + 2) % 3 + 1] = MI_PIXEL(mi,
 			NRAND(MI_NPIXELS(mi) / COLORBASE) +
 			2 * MI_NPIXELS(mi) / COLORBASE);
+#else
+		int j = NRAND(MI_NPIXELS(mi));
+		i = NRAND(3);
+		lp->colors[i + 1] =  MI_PIXEL(mi, j);
+		lp->colors[(i + 1) % 3 + 1] =
+			MI_PIXEL(mi, (j + 2) % MI_NPIXELS(mi));
+		lp->colors[(i + 2) % 3 + 1] =
+			MI_PIXEL(mi, (j + 4) % MI_NPIXELS(mi));
+#endif
 	} else {
 		/*lp->colors[1] = lp->colors[2] = lp->colors[3] =
 			MI_WHITE_PIXEL(mi);*/
