@@ -169,7 +169,7 @@ static int  descr();
 #ifdef USE_MB
 # define DEF_FONTSET	DEF_FONT ",-*-*-medium-r-normal--24-*"
 # define DEF_PLANFONTSET DEF_PLANFONT ",-*-*-medium-r-normal--14-*"
-#endif /* USE MB */
+#endif /* USE NONASCII */
 #define DEF_BG		"White"
 #define DEF_FG		"Black"
 
@@ -620,8 +620,8 @@ static OptionStruct opDesc[] =
 	{(char *) "-fpsfont fontname", (char *) "font to use for FPS display"},
 #endif
 #ifdef USE_MB
-	{(char *) "-fontset fontsetname", (char *) "fontset to use for Xmb..."},
-	{(char *) "-planfontset fontsetname", (char *) "fontset to use for Xmb..."},
+	{(char *) "-fontset fontsetname", (char *) "fontset to use for Xmb"},
+	{(char *) "-planfontset fontsetname", (char *) "fontset to use for Xmb"},
 #endif
 	{(char *) "-bg color", (char *) "background color to use for password prompt"},
 	{(char *) "-fg color", (char *) "foreground color to use for password prompt"},
@@ -630,11 +630,11 @@ static OptionStruct opDesc[] =
 	{(char *) "-username string", (char *) "text string to use for Name prompt"},
 	{(char *) "-password string", (char *) "text string to use for Password prompt"},
 	{(char *) "-info string", (char *) "text string to use for instructions"},
-  {(char *) "-validate string", (char *) "text string to use for validating password message"},
-      {(char *) "-invalid string", (char *) "text string to use for invalid password message"},
-      {(char *) "-invalidCapsLock string", (char *) "text string to use for invalid password message with Caps Lock on"},
+	{(char *) "-validate string", (char *) "text string to use for validating password message"},
+	{(char *) "-invalid string", (char *) "text string to use for invalid password message"},
+	{(char *) "-invalidCapsLock string", (char *) "text string to use for invalid password message with Caps Lock on"},
 	{(char *) "-geometry geom", (char *) "geometry for non-full screen lock"},
-   {(char *) "-icongeometry geom", (char *) "geometry for password window (location ignored)"},
+	{(char *) "-icongeometry geom", (char *) "geometry for password window (location ignored)"},
 #ifdef FX
 	{(char *) "-glgeometry geom", (char *) "geometry for gl modes (location ignored)"},
 #endif
@@ -2190,9 +2190,9 @@ getResources(Display ** displayp, int argc, char **argv)
 		}
 		images_list = (struct dirent ***) malloc(sizeof (struct dirent **));
 
-		num_list = scan_dir(directory_r, images_list, sel_image, NULL);
-		image_list = *images_list;
 		if (images_list) {
+			num_list = scan_dir(directory_r, images_list, sel_image, NULL);
+			image_list = *images_list;
 			free(images_list);
 			images_list = (struct dirent ***) NULL;
 		}
